﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using MercatorUi;
using MercatorTunnel;
using MercatorApi;

namespace ManageTable
{
    public partial class GestionTableForm : MercatorUi._BaseClasses.BaseForm, MercatorUi.Interfaces.IExternalForm
    {
        public string retMsg = "";
        private bool Saved = false;
        private List<string> lstRecordDeleted = new List<string>();
        private string TableNom;
        private DataTable dtLignes;
        private Int16 Type = 0;
        private string TypeChar = ""; 
        private StringBuilder LogMsg = new StringBuilder();

        public GestionTableForm(string NomTable, string ReqSql, string Libelle, Int16 type, string typeChar)
        {
            TableNom = NomTable;
            Type = type;
            if (typeChar != "")
            {
                // Si le champ Type de la table est de type caractère, alors, traitement différent
                TypeChar = typeChar;
                Type = -1;
            }

            InitializeComponent();

            Text = "Gestion de " + Libelle.ToString().Trim();
            ReqSql = ReqSql.ToUpper().Replace("TIMESTAMP_COLUMN", "");
            ReqSql += string.Format(@" select column_name, data_type 
                                            from information_schema.columns 
                                            where table_name = '{0}' 
                                                and column_name in ('Id', 'Nom')
                                            order by column_name ",Api.UnquoteSql(TableNom));

            DataSet dsLignes = Api.Zselect(Globals.RepData, ReqSql.ToString());
            if (dsLignes != null)
            {
                if ((dsLignes.Tables[1].Rows.Count == 2) && (dsLignes.Tables[1].Rows[0]["Data_type"].ToString().Trim() == "char") && (dsLignes.Tables[1].Rows[1]["Data_type"].ToString().Trim() == "char"))
                {
                    Api.TrimEndDataSet(dsLignes);
                    dtLignes = dsLignes.Tables[0];
                    dtLignes.AcceptChanges();

                    Lignes.AutoGenerateColumns = true;
                    Lignes.AllowUserToAddRows = true;
                    Lignes.AllowFilter = false;
                    Lignes.DataSource = dtLignes;
                }
                else
                {
                    Dialogs.Stop("Il est obligatoire d'avoir \r\n   - un champ [id] identifiant\r\n   - un champ [nom].\r\n\r\nTous deux\r\n   - de type caractère \r\n   - en 10 positions !");
                    btnOk.Enabled = false;
                    btnLignes.Enabled = false;
                    Lignes.Enabled = false;
                }
            }
            else
            {
                btnOk.Enabled = false;
                btnLignes.Enabled = false;
                Lignes.Enabled = false;
            }
            
        }

        private void Gestion_Load(object sender, EventArgs e)
        {
            //supprimer le champ timestamp de la datatable
            try
            {
                dtLignes.Columns.Remove("timestamp_column");
            }
            catch (Exception ex) 
            {
                LogMsg.AppendLine(ex.Message.ToString());
            }


            Lignes.AutoGenerateColumns = true;
            //associer la datatable à la grille 
            try
            {
                Lignes.DataSource = dtLignes;
                
            }
            catch (Exception ex)
            {
                Dialogs.Stop(ex.Message);
                retMsg = "Erreur lors de l'association de la table à la grille !";
                return;
            }

            //masquer colonne type

            if (dtLignes.Columns.Contains("type"))
            {
                try
                {
                    Lignes.Columns["type"].Visible = false;
                }
                catch (Exception ex) { LogMsg.AppendLine(ex.Message.ToString()); }
            }
            //masquer colonne id
            try
            {
                Lignes.Columns["id"].Visible = false;
            }
            catch (Exception ex) { LogMsg.AppendLine(ex.Message.ToString()); }


            this.CenterToScreen();
        }

        private void btnOk_Click(object sender, EventArgs e)
        {

            if (!isModified())
            {
                Close();
                return;
            }

            MercatorSqlConnection oConnSql = new MercatorSqlConnection(MercatorUi.Globals.RepData, true);
            if (oConnSql == null)
                return;
            using (SqlTransaction transac = oConnSql.Connection.BeginTransaction())
            {
                using (SqlCommand oCommand = new SqlCommand(string.Format("delete from {0} where id = @ID", TableNom), oConnSql.Connection, transac))
                {
                    foreach (string id in lstRecordDeleted)
                    {
                        oCommand.Parameters.Add("@ID", SqlDbType.Char).Value = id;
                        Api.ZstoperrorString2 = _Divers.Iif_langue(MercatorUi.Globals.Langue, "Impossible to delete this line : ", "Onmogelijk deze lijn te verwijderen : ", "Impossible de supprimer cette ligne : ") + id;
                        if (!Api.SqlExec(oCommand))
                        {
                            Api.SafeRollback(transac);
                            oConnSql.Connection.Close();
                            oConnSql.Connection.Dispose();
                            return;
                        }
                        oCommand.Parameters.Clear();
                    }
                }

                StringBuilder FieldListUpdate = new StringBuilder();
                StringBuilder FieldListInsertFields = new StringBuilder();
                StringBuilder FieldListInsertValues = new StringBuilder();

                foreach (DataColumn dc in dtLignes.Columns)
                {
                    // Liste de champs pour l'update
                    if (FieldListUpdate.ToString() != "")
                        FieldListUpdate.Append(",");
                    FieldListUpdate.Append(dc.ColumnName + " = @" + dc.ColumnName.ToUpper());

                    // Liste de champs pour l'insert
                    if (FieldListInsertFields.ToString() != "")
                        FieldListInsertFields.Append(",");
                    FieldListInsertFields.Append(dc.ColumnName);

                    if (FieldListInsertValues.ToString() != "")
                        FieldListInsertValues.Append(",");
                    FieldListInsertValues.Append("@" + dc.ColumnName.ToUpper());
                }

                StringBuilder ReqSql = new StringBuilder();
                ReqSql.AppendFormat("if exists(select id from {0} where id = @ID)", TableNom);
                ReqSql.AppendLine("\r\n");
                ReqSql.AppendFormat("     update {0} set {1} where id = @ID ", TableNom, FieldListUpdate.ToString());
                ReqSql.AppendLine("\r\n");
                ReqSql.AppendLine(" else ");
                ReqSql.AppendLine("\r\n");
                ReqSql.AppendFormat("     insert into {0} ({1}) values ({2})", TableNom, FieldListInsertFields.ToString(), FieldListInsertValues.ToString());

                using (SqlCommand oCommand = new SqlCommand(ReqSql.ToString(), oConnSql.Connection, transac))
                {
                    foreach (DataRow dr in dtLignes.Rows)
                    {
                        if ((dr["nom"] == DBNull.Value) || (dr["nom"].ToString().Trim() == ""))
                            continue;
                        if (!((dr.RowState == DataRowState.Added) || (dr.RowState == DataRowState.Modified)))
                            continue;
                        if ((dr["id"] == DBNull.Value) || (dr["id"].ToString().Trim() == ""))
                            dr["id"] = Api.Ident();

                        if (dtLignes.Columns.Contains("type"))
                        {
                            if (Type != -1)
                            {
                                if ((this.Type > 0) && ((dr["type"] == DBNull.Value) || (Convert.ToInt16(dr["type"]) != this.Type)))
                                    dr["type"] = this.Type;
                            }
                            else
                            {
                                if ((this.TypeChar != "") && ((dr["type"] == DBNull.Value) || (dr["type"].ToString().Trim() != this.TypeChar)))
                                    dr["type"] = this.TypeChar;
                            }
                        }
                        foreach (DataColumn dc in dtLignes.Columns)
                        {
                            oCommand.Parameters.Add("@" + dc.ColumnName.ToUpper(), SqlDbType.Char).Value = dr[dc.ColumnName];
                        }

                        Api.ZstoperrorString2 = _Divers.Iif_langue(Globals.Langue, "Impossible to add this line : ", "Onmogelijk deze lijn bij te voegen : ", "Impossible d'ajouter cette ligne : ") + dr["nom"].ToString();
                        if (!Api.SqlExec(oCommand))
                        {
                            Api.SafeRollback(transac);
                            oConnSql.Connection.Close();
                            oConnSql.Connection.Dispose();
                            return;
                        }
                        oCommand.Parameters.Clear();
                    }
                }
                transac.Commit();
                oConnSql.Connection.Close();
                oConnSql.Connection.Dispose();

                Saved = true;
                Close();
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            // Annulation
            Close();
        }

        private void Gestion_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (isModified() && !Dialogs.AnswerYesNo(Api.Iif_langue(Globals.Langue, 8)))
                e.Cancel = true;
        }
        private bool isModified()
        {
            bool RetVal = false;

            if (! Saved)
            {
                if (dtLignes != null)
                {
                    foreach (DataRow dr in dtLignes.Rows)
                    {
                        if ((dr.RowState == DataRowState.Added) || (dr.RowState == DataRowState.Modified))
                            RetVal = true;
                    }
                }
                if (lstRecordDeleted.Count > 0)
                    RetVal = true;
            }
            return RetVal;
        }

        private DataRow New_Row(DataTable dt)
        {
            DataRow dr = dt.NewRow();
            if (dtLignes.Columns.Contains("type"))
            {
                try
                {
                    if (Type != -1)
                    {
                        if ((this.Type > 0) && ((dr["type"] == DBNull.Value) || (Convert.ToInt16(dr["type"]) != this.Type)))
                            dr["type"] = this.Type;
                    }
                    else
                    {
                        if ((this.TypeChar != "") && ((dr["type"] == DBNull.Value) || (dr["type"].ToString().Trim() != this.TypeChar)))
                            dr["type"] = this.TypeChar;
                    }
                }
                catch (Exception ex) { LogMsg.AppendLine(ex.Message.ToString()); }
            }
            try
            {
                if ((dr["id"] == DBNull.Value) || (dr["id"].ToString().Trim() == ""))
                    dr["id"] = Api.Ident();
            }
            catch (Exception ex) { LogMsg.AppendLine(ex.Message.ToString()); }
            
            return dr;
        }

        private void btnLignesAdd_Execute(object sender, EventArgs e)
        {
            dtLignes.Rows.Add(New_Row(dtLignes));
        }
        private void btnLignesDel_Execute(object sender, EventArgs e)
        {
            if ((Lignes.SelectedCells != null) && (Lignes.SelectedCells.Count > 0))
            {
                if (Lignes.SelectedCells[0].RowIndex < dtLignes.Rows.Count)
                {
                    string id = "";
                    if (dtLignes.Rows[Lignes.SelectedCells[0].RowIndex]["id"] != DBNull.Value)
                        id = dtLignes.Rows[Lignes.SelectedCells[0].RowIndex]["id"].ToString();
                    if (id != "")
                        lstRecordDeleted.Add(id);
                    dtLignes.Rows.Remove(dtLignes.Rows[Lignes.SelectedCells[0].RowIndex]);
                }
            }
        }

        private void Lignes_KeyPress(object sender, KeyPressEventArgs e)
        {
            //accepter les caractères backspace,del,...
            if (Char.IsControl(e.KeyChar))
                return;

            Type type_cell = Lignes.CurrentCell.ValueType;
            if (type_cell == typeof(Double))
            {
                //forcer la virgule pour le point du clavier numérique
                string sepdec = System.Globalization.CultureInfo.CurrentCulture.NumberFormat.NumberDecimalSeparator;
                if ((sepdec == ",") && (e.KeyChar == Convert.ToChar(".")))
                    e.KeyChar = Convert.ToChar(",");

                //refuser les caractères non numériques
                if ((!char.IsNumber(e.KeyChar) || (e.KeyChar == '²') || (e.KeyChar == '³')) && (e.KeyChar != Convert.ToChar(",")) && (e.KeyChar != Convert.ToChar(".")))
                    e.Handled = true;

                //refuser une seconde virgule
                if ((Lignes.CurrentCell.EditedFormattedValue.ToString().IndexOf(',') > 0) && ((e.KeyChar == Convert.ToChar(",")) || (e.KeyChar == Convert.ToChar("."))))
                    e.Handled = true;
            }

            if ((type_cell == typeof(Int16)) || (type_cell == typeof(Int32)) || (type_cell == typeof(Int64)))
            {
                //refuser les caractères non numériques
                if (!char.IsNumber(e.KeyChar) || (e.KeyChar == '²'))
                    e.Handled = true;
            }
        }

        private void Lignes_Resize(object sender, EventArgs e)
        {
            this.btnOk.Left = this.Width - 153;
            this.btnCancel.Left = this.Width - 285;
            this.btnLignes.Left = 5;
        }
    }
}
